@@ -164,6 +164,9 @@
  */
 #define NX_HMI_ACTIVE		PPC_BIT(54)
 
+/* Number of iterations for the various timeouts */
+#define TIMEOUT_LOOPS		20000000
+
 static const struct core_xstop_bit_info {
 	uint8_t bit;		/* CORE FIR bit number */
 	enum OpalHMI_CoreXstopReason reason;
@@ -253,13 +256,15 @@ static int handle_capp_recoverable(int chip_id)
 	u32 dt_chip_id;
 	struct phb *phb;
 	u32 phb_index;
+	struct proc_chip *chip = get_chip(chip_id);
+	u8 mask = chip->capp_phb3_attached_mask;
 
 	dt_for_each_compatible(dt_root, np, "ibm,power8-pciex") {
 		dt_chip_id = dt_prop_get_u32(np, "ibm,chip-id");
 		phb_index = dt_prop_get_u32(np, "ibm,phb-index");
 		phb_id = dt_prop_get_u64(np, "ibm,opal-phbid");
 
-		if ((phb_index == 0) && (chip_id == dt_chip_id)) {
+		if ((mask & (1 << phb_index)) && (chip_id == dt_chip_id)) {
 			phb = pci_get_phb(phb_id);
 			phb->ops->lock(phb);
 			phb->ops->set_capp_recovery(phb);
@@ -403,6 +408,17 @@ static void find_nx_checkstop_reason(int flat_chip_id,
 			hmi_evt->u.xstop_error.xstop_reason
 						|= nx_pbi_xstop_bits[i].reason;
 
+	/*
+	 * Set NXDMAENGFIR[38] to signal PRD that service action is required.
+	 * Without this inject, PRD will not be able to do NX unit checkstop
+	 * error analysis. NXDMAENGFIR[38] is a spare bit and used to report
+	 * a software initiated attention.
+	 *
+	 * The behavior of this bit and all FIR bits are documented in
+	 * RAS spreadsheet.
+	 */
+	xscom_write(flat_chip_id, NX_DMA_ENGINE_FIR, PPC_BIT(38));
+
 	/* Send an HMI event. */
 	queue_hmi_event(hmi_evt, 0);
 	*event_generated = 1;
@@ -448,8 +464,27 @@ static int decode_malfunction(struct OpalHMIEvent *hmi_evt)
 
 static void wait_for_subcore_threads(void)
 {
-	while (!(*(this_cpu()->core_hmi_state_ptr) & HMI_STATE_CLEANUP_DONE))
+	uint64_t timeout = 0;
+
+	while (!(*(this_cpu()->core_hmi_state_ptr) & HMI_STATE_CLEANUP_DONE)) {
+		/*
+		 * We use a fixed number of TIMEOUT_LOOPS rather
+		 * than using the timebase to do a pseudo-wall time
+		 * timeout due to the fact that timebase may not actually
+		 * work at this point in time.
+		 */
+		if (++timeout >= (TIMEOUT_LOOPS*3)) {
+			/*
+			 * Break out the loop here and fall through
+			 * recovery code. If recovery fails, kernel will get
+			 * informed about the failure. This way we can avoid
+			 * looping here if other threads are stuck.
+			 */
+			prlog(PR_DEBUG, "HMI: TB pre-recovery timeout\n");
+			break;
+		}
 		cpu_relax();
+	}
 }
 
 /*
@@ -610,7 +645,13 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 	pre_recovery_cleanup();
 
 	lock(&hmi_lock);
-	printf("HMI: Received HMI interrupt: HMER = 0x%016llx\n", hmer);
+	/*
+	 * Not all HMIs would move TB into invalid state. Set the TB state
+	 * looking at TFMR register. TFMR will tell us correct state of
+	 * TB register.
+	 */
+	this_cpu()->tb_invalid = !(mfspr(SPR_TFMR) & SPR_TFMR_TB_VALID);
+	prlog(PR_DEBUG, "HMI: Received HMI interrupt: HMER = 0x%016llx\n", hmer);
 	if (hmi_evt)
 		hmi_evt->hmer = hmer;
 	if (hmer & SPR_HMER_PROC_RECV_DONE) {
@@ -620,7 +661,7 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 			hmi_evt->type = OpalHMI_ERROR_PROC_RECOV_DONE;
 			queue_hmi_event(hmi_evt, recover);
 		}
-		printf("HMI: Processor recovery Done.\n");
+		prlog(PR_DEBUG, "HMI: Processor recovery Done.\n");
 	}
 	if (hmer & SPR_HMER_PROC_RECV_ERROR_MASKED) {
 		hmer &= ~SPR_HMER_PROC_RECV_ERROR_MASKED;
@@ -629,7 +670,7 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 			hmi_evt->type = OpalHMI_ERROR_PROC_RECOV_MASKED;
 			queue_hmi_event(hmi_evt, recover);
 		}
-		printf("HMI: Processor recovery Done (masked).\n");
+		prlog(PR_DEBUG, "HMI: Processor recovery Done (masked).\n");
 	}
 	if (hmer & SPR_HMER_PROC_RECV_AGAIN) {
 		hmer &= ~SPR_HMER_PROC_RECV_AGAIN;
@@ -638,7 +679,7 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 			hmi_evt->type = OpalHMI_ERROR_PROC_RECOV_DONE_AGAIN;
 			queue_hmi_event(hmi_evt, recover);
 		}
-		printf("HMI: Processor recovery occurred again before"
+		prlog(PR_DEBUG, "HMI: Processor recovery occurred again before"
 			"bit2 was cleared\n");
 	}
 	/* Assert if we see malfunction alert, we can not continue. */
@@ -697,6 +738,8 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 	 */
 	mtspr(SPR_HMER, hmer);
 	hmi_exit();
+	/* Set the TB state looking at TFMR register before we head out. */
+	this_cpu()->tb_invalid = !(mfspr(SPR_TFMR) & SPR_TFMR_TB_VALID);
 	unlock(&hmi_lock);
 	return recover;
 }
@@ -704,7 +747,6 @@ int handle_hmi_exception(uint64_t hmer, struct OpalHMIEvent *hmi_evt)
 static int64_t opal_handle_hmi(void)
 {
 	uint64_t hmer;
-	int rc = OPAL_SUCCESS;
 	struct OpalHMIEvent hmi_evt;
 
 	/*
@@ -719,6 +761,6 @@ static int64_t opal_handle_hmi(void)
 	hmer = mfspr(SPR_HMER);		/* Get HMER register value */
 	handle_hmi_exception(hmer, &hmi_evt);
 
-	return rc;
+	return OPAL_SUCCESS;
 }
 opal_call(OPAL_HANDLE_HMI, opal_handle_hmi, 0);
