@@ -45,12 +45,6 @@
 #include <ipmi.h>
 #include <sensor.h>
 
-/*
- * Boot semaphore, incremented by each CPU calling in
- *
- * Forced into data section as it will be used before BSS is initialized
- */
-enum ipl_state ipl_state = ipl_initial;
 enum proc_gen proc_gen;
 
 static uint64_t kernel_entry;
@@ -63,6 +57,7 @@ void skiboot_gcov_done(void);
 struct debug_descriptor debug_descriptor = {
 	.eye_catcher	= "OPALdbug",
 	.version	= DEBUG_DESC_VERSION,
+	.state_flags	= 0,
 	.memcons_phys	= (uint64_t)&memcons,
 	.trace_mask	= 0, /* All traces disabled by default */
 	.console_log_levels = (PR_DEBUG << 4) | PR_NOTICE,
@@ -460,6 +455,8 @@ void __noreturn load_and_boot_kernel(bool is_reboot)
 	printf("INIT: Starting kernel at 0x%llx, fdt at %p (size 0x%x)\n",
 	       kernel_entry, fdt, fdt_totalsize(fdt));
 
+	debug_descriptor.state_flags |= OPAL_BOOT_COMPLETE;
+
 	fdt_set_boot_cpuid_phys(fdt, this_cpu()->pir);
 	if (kernel_32bit)
 		start_kernel32(kernel_entry, fdt, mem_top);
@@ -756,7 +753,7 @@ void __noreturn main_cpu_entry(const void *fdt, u32 master_cpu)
 	 * because they rely on various other things having been setup,
 	 * for example, add_opal_interrupts() will add all the interrupt
 	 * sources that are going to the firmware. We can't add a new one
-	 * after that call. Similarily, the mem_region calls will construct
+	 * after that call. Similarly, the mem_region calls will construct
 	 * the reserve maps in the DT so we shouldn't affect the memory
 	 * regions after that
 	 */
