@@ -14,6 +14,7 @@
  * limitations under the License.
  */
 
+#include <inttypes.h>
 #include <skiboot.h>
 #include <mem-map.h>
 #include <libfdt_env.h>
@@ -133,7 +134,8 @@ static struct alloc_hdr *next_hdr(const struct mem_region *region,
 static void init_allocatable_region(struct mem_region *region)
 {
 	struct free_hdr *f = region_start(region);
-	assert(region->type == REGION_SKIBOOT_HEAP);
+	assert(region->type == REGION_SKIBOOT_HEAP ||
+	       region->type == REGION_MEMORY);
 	f->hdr.num_longs = region->len / sizeof(long);
 	f->hdr.free = true;
 	f->hdr.prev_free = false;
@@ -256,11 +258,16 @@ static void bad_header(const struct mem_region *region,
 	abort();
 }
 
-static bool region_is_reserved(struct mem_region *region)
+static bool region_is_reservable(struct mem_region *region)
 {
 	return region->type != REGION_OS;
 }
 
+static bool region_is_reserved(struct mem_region *region)
+{
+	return region->type != REGION_OS && region->type != REGION_MEMORY;
+}
+
 void mem_dump_allocs(void)
 {
 	struct mem_region *region;
@@ -269,7 +276,8 @@ void mem_dump_allocs(void)
 	/* Second pass: populate property data */
 	printf("Memory regions:\n");
 	list_for_each(&regions, region, list) {
-		if (region->type != REGION_SKIBOOT_HEAP)
+		if (!(region->type == REGION_SKIBOOT_HEAP ||
+		      region->type == REGION_MEMORY))
 			continue;
 		printf("  0x%012llx..%012llx : %s\n",
 		       (long long)region->start,
@@ -299,7 +307,8 @@ int64_t mem_dump_free(void)
 
 	printf("Free space in HEAP memory regions:\n");
 	list_for_each(&regions, region, list) {
-		if (region->type != REGION_SKIBOOT_HEAP)
+		if (!(region->type == REGION_SKIBOOT_HEAP ||
+		      region->type == REGION_MEMORY))
 			continue;
 		region_free = 0;
 
@@ -312,11 +321,12 @@ int64_t mem_dump_free(void)
 
 			region_free+= hdr->num_longs * sizeof(long);
 		}
-		printf("Region %s free: %llu\n", region->name, region_free);
+		printf("Region %s free: %"PRIx64"\n",
+		       region->name, region_free);
 		total_free += region_free;
 	}
 
-	printf("Total free: %llu\n", total_free);
+	printf("Total free: %"PRIu64"\n", total_free);
 
 	return total_free;
 }
@@ -335,7 +345,8 @@ static void *__mem_alloc(struct mem_region *region, size_t size, size_t align,
 	assert(is_rodata(location));
 
 	/* Unallocatable region? */
-	if (region->type != REGION_SKIBOOT_HEAP)
+	if (!(region->type == REGION_SKIBOOT_HEAP ||
+	      region->type == REGION_MEMORY))
 		return NULL;
 
 	/* First allocation? */
@@ -535,8 +546,9 @@ bool mem_check(const struct mem_region *region)
 	}
 
 	/* Not ours to play with, or empty?  Don't do anything. */
-	if (region->type != REGION_SKIBOOT_HEAP ||
-			region->free_list.n.next == NULL)
+	if (!(region->type == REGION_MEMORY ||
+	      region->type == REGION_SKIBOOT_HEAP) ||
+	    region->free_list.n.next == NULL)
 		return true;
 
 	/* Walk linearly. */
@@ -676,7 +688,7 @@ static bool add_region(struct mem_region *region)
 	struct mem_region *r;
 
 	if (mem_regions_finalised) {
-		prerror("MEM: add_region(%s@0x%llx) called after finalise!\n",
+		prerror("MEM: add_region(%s@0x%"PRIx64") called after finalise!\n",
 				region->name, region->start);
 		return false;
 	}
@@ -738,7 +750,8 @@ restart:
 		const struct dt_property *prop;
 		const __be32 *ids;
 
-		if (region->type != REGION_SKIBOOT_HEAP)
+		if (!(region->type == REGION_SKIBOOT_HEAP ||
+		      region->type == REGION_MEMORY))
 			continue;
 
 		/* Don't allocate from normal heap. */
@@ -766,7 +779,7 @@ restart:
 
 	/*
 	 * If we can't allocate the memory block from the expected
-	 * node, we bail to any one that can accomodate our request.
+	 * node, we bail to any one that can accommodate our request.
 	 */
 	if (!p && use_local) {
 		use_local = false;
@@ -945,11 +958,12 @@ void mem_region_init(void)
 		if (!dt_has_node_property(i, "device_type", "memory"))
 			continue;
 		rname = zalloc(strlen(i->name) + strlen(NODE_REGION_PREFIX) + 1);
+		assert(rname);
 		strcat(rname, NODE_REGION_PREFIX);
 		strcat(rname, i->name);
 		start = dt_get_address(i, 0, &len);
 		lock(&mem_region_lock);
-		region = new_region(rname, start, len, i, REGION_SKIBOOT_HEAP);
+		region = new_region(rname, start, len, i, REGION_MEMORY);
 		if (!region) {
 			prerror("MEM: Could not add mem region %s!\n", i->name);
 			abort();
@@ -1022,7 +1036,8 @@ void mem_region_release_unused(void)
 		uint64_t used_len;
 
 		/* If it's not allocatable, ignore it. */
-		if (r->type != REGION_SKIBOOT_HEAP)
+		if (!(r->type == REGION_SKIBOOT_HEAP ||
+		      r->type == REGION_MEMORY))
 			continue;
 
 		used_len = allocated_length(r);
@@ -1078,6 +1093,7 @@ static void mem_region_add_dt_reserved_node(struct dt_node *parent,
 	}
 
 	name = strdup(region->name);
+	assert(name);
 
 	/* remove any cell addresses in the region name; we have our own cell
 	 * addresses here */
@@ -1086,6 +1102,7 @@ static void mem_region_add_dt_reserved_node(struct dt_node *parent,
 		*p = '\0';
 
 	region->node = dt_new_addr(parent, name, region->start);
+	assert(region->node);
 	dt_add_property_u64s(region->node, "reg", region->start, region->len);
 	free(name);
 }
@@ -1120,7 +1137,7 @@ void mem_region_add_dt_reserved(void)
 
 	/* First pass: calculate length of property data */
 	list_for_each(&regions, region, list) {
-		if (!region_is_reserved(region))
+		if (!region_is_reservable(region))
 			continue;
 		names_len += strlen(region->name) + 1;
 		ranges_len += 2 * sizeof(uint64_t);
@@ -1132,7 +1149,7 @@ void mem_region_add_dt_reserved(void)
 	printf("Reserved regions:\n");
 	/* Second pass: populate property data */
 	list_for_each(&regions, region, list) {
-		if (!region_is_reserved(region))
+		if (!region_is_reservable(region))
 			continue;
 		len = strlen(region->name) + 1;
 		memcpy(name, region->name, len);
