@@ -20,6 +20,7 @@
 #include <pci-cfg.h>
 #include <timebase.h>
 #include <device.h>
+#include <fsp.h>
 
 /* The eeh event code will need updating if this is ever increased to
  * support more than 64 phbs */
@@ -608,20 +609,23 @@ static int pci_configure_mps(struct phb *phb,
 			     struct pci_device *pd,
 			     void *userdata __unused)
 {
-	uint32_t ecap, mps;
+	uint32_t ecap, aercap, mps;
 	uint16_t val;
 
 	assert(phb);
 	assert(pd);
 
-	mps = phb->mps;
 	/* If the MPS isn't acceptable one, bail immediately */
+	mps = phb->mps;
 	if (mps < 128 || mps > 4096)
 		return 1;
 
+	/* Retrieve PCIe and AER capability */
+	ecap = pci_cap(pd, PCI_CFG_CAP_ID_EXP, false);
+	aercap = pci_cap(pd, PCIECAP_ID_AER, true);
+
 	/* PCIe device always has MPS capacity */
 	if (pd->mps) {
-		ecap = pci_cap(pd, PCI_CFG_CAP_ID_EXP, false);
 		mps = ilog2(mps) - 7;
 
 		pci_cfg_read16(phb, pd->bdfn, ecap + PCICAP_EXP_DEVCTL, &val);
@@ -629,6 +633,19 @@ static int pci_configure_mps(struct phb *phb,
 		pci_cfg_write16(phb, pd->bdfn, ecap + PCICAP_EXP_DEVCTL, val);
 	}
 
+	/* Changing MPS on upstream PCI bridge might cause some error
+	 * bits in PCIe and AER capability. To clear them to avoid
+	 * confusion.
+	 */
+	if (aercap) {
+		pci_cfg_write32(phb, pd->bdfn, aercap + PCIECAP_AER_UE_STATUS,
+				0xffffffff);
+		pci_cfg_write32(phb, pd->bdfn, aercap + PCIECAP_AER_CE_STATUS,
+				0xffffffff);
+	}
+	if (ecap)
+		pci_cfg_write16(phb, pd->bdfn, ecap + PCICAP_EXP_DEVSTAT, 0xf);
+
 	return 0;
 }
 
@@ -1122,34 +1139,44 @@ static void pci_add_slot_properties(struct phb *phb, struct pci_slot_info *info,
 				    struct dt_node *np)
 {
 	char loc_code[LOC_CODE_SIZE];
-	size_t base_loc_code_len, slot_label_len;
+	size_t base_loc_code_len = 0, slot_label_len = 0;
 
 	if (phb->base_loc_code) {
 		base_loc_code_len = strlen(phb->base_loc_code);
-		slot_label_len = strlen(info->label);
-		if ((base_loc_code_len + slot_label_len +1) < LOC_CODE_SIZE) {
-			strcpy(loc_code, phb->base_loc_code);
-			strcat(loc_code, "-");
+		strcpy(loc_code, phb->base_loc_code);
+	}
+	slot_label_len = strlen(info->label);
+	if (slot_label_len) {
+		if ((base_loc_code_len + slot_label_len + 1) < LOC_CODE_SIZE) {
+			if (base_loc_code_len)
+				strcat(loc_code, "-");
 			strcat(loc_code, info->label);
 			dt_add_property(np, "ibm,slot-location-code",
 					loc_code, strlen(loc_code) + 1);
-		} else
+		} else {
 			PCIERR(phb, 0, "Loc Code too long - %zu + %zu + 1\n",
 			       base_loc_code_len, slot_label_len);
-	} else
-		PCIERR(phb, 0, "Base Loc code not found...\n");
+		}
+	}
 
 	/* Add other slot information */
 	dt_add_property_cells(np, "ibm,slot-pluggable", info->pluggable);
 	dt_add_property_cells(np, "ibm,slot-power-ctl", info->power_ctl);
-	dt_add_property_cells(np, "ibm,slot-wired-lanes", info->wired_lanes);
+	if (info->wired_lanes >= 0)
+		dt_add_property_cells(np, "ibm,slot-wired-lanes", info->wired_lanes);
 	/*dt_add_property(np, "ibm,slot-bus-clock", &pd->slot_info->bus_clock, sizeof(uint8_t));*/
-	dt_add_property_cells(np, "ibm,slot-connector-type", info->connector_type);
-	dt_add_property_cells(np, "ibm,slot-card-desc", info->card_desc);
-	dt_add_property_cells(np, "ibm,slot-card-mech", info->card_mech);
-	dt_add_property_cells(np, "ibm,slot-pwr-led-ctl", info->pwr_led_ctl);
-	dt_add_property_cells(np, "ibm,slot-attn-led-ctl", info->attn_led_ctl);
-	dt_add_property_string(np, "ibm,slot-label", info->label);
+	if (info->connector_type >= 0)
+		dt_add_property_cells(np, "ibm,slot-connector-type", info->connector_type);
+	if (info->card_desc >= 0)
+		dt_add_property_cells(np, "ibm,slot-card-desc", info->card_desc);
+	if (info->card_mech >= 0)
+		dt_add_property_cells(np, "ibm,slot-card-mech", info->card_mech);
+	if (info->pwr_led_ctl >= 0)
+		dt_add_property_cells(np, "ibm,slot-pwr-led-ctl", info->pwr_led_ctl);
+	if (info->attn_led_ctl >= 0)
+		dt_add_property_cells(np, "ibm,slot-attn-led-ctl", info->attn_led_ctl);
+	if (strlen(info->label) > 0)
+		dt_add_property_string(np, "ibm,slot-label", info->label);
 }
 
 static void pci_add_loc_code(struct dt_node *np, struct pci_device *pd)
@@ -1173,7 +1200,11 @@ static void pci_add_loc_code(struct dt_node *np, struct pci_device *pd)
 	class_code = dt_prop_get_u32(np, "class-code");
 	class = class_code >> 16;
 	sub = (class_code >> 8) & 0xff;
-	if (class == 0x02 && sub == 0x00) {
+
+	/* XXX Don't do that on openpower for now, we will need to sort things
+	 * out later, otherwise the mezzanine slot on Habanero gets weird results
+	 */
+	if (class == 0x02 && sub == 0x00 && fsp_present()) {
 		/* There's usually several spaces at the end of the property.
 		   Test for, but don't rely on, that being the case */
 		len = strlen(blcode);
