@@ -31,13 +31,20 @@ static void init_rand(void)
 	}
 }
 
-static void expiry(struct timer *t, void *data)
+static void expiry(struct timer *t, void *data, uint64_t now)
 {
 	(void)data;
+	(void)now;
 	assert(t->target >= last);
 	count--;
 }
 
+void slw_update_timer_expiry(uint64_t new_target)
+{
+	(void)new_target;
+	/* FIXME: do intersting SLW timer sim */
+}
+
 int main(void)
 {
 	unsigned int i;
