@@ -40,6 +40,8 @@
 
 #include "gard.h"
 
+#define CLEARED_RECORD_ID 0xFFFFFFFF
+
 #define FDT_ACTIVE_FLASH_PATH "/proc/device-tree/chosen/ibm,system-flash"
 #define SYSFS_MTD_PATH "/sys/class/mtd/"
 #define FLASH_GARD_PART "GUARD"
@@ -277,12 +279,13 @@ static int get_dev_mtd(const char *fdt_flash_path, char **r_path)
 		return -1;
 	}
 
-	rc = read(fd, fdt_node_path, sizeof(fdt_node_path));
+	rc = read(fd, fdt_node_path, sizeof(fdt_node_path) - 1);
 	close(fd);
 	if (rc == -1) {
 		fprintf(stderr, "Couldn't read flash FDT node from '%s'\n", fdt_flash_path);
 		return -1;
 	}
+	fdt_node_path[rc] = '\0';
 
 	count = scandir(SYSFS_MTD_PATH, &namelist, NULL, alphasort);
 	if (count == -1) {
@@ -358,6 +361,11 @@ static int get_dev_mtd(const char *fdt_flash_path, char **r_path)
 	return done ? rc : -1;
 }
 
+static bool is_valid_id(uint32_t record_id)
+{
+	return record_id != CLEARED_RECORD_ID;
+}
+
 static int do_iterate(struct gard_ctx *ctx,
                       int (*func)(struct gard_ctx *ctx, int pos,
                                   struct gard_record *gard, void *priv),
@@ -404,13 +412,36 @@ static int get_largest_pos(struct gard_ctx *ctx)
 	return largest;
 }
 
+static int count_valid_records_i(struct gard_ctx *ctx, int pos, struct gard_record *gard, void *priv)
+{
+	if (!gard || !priv)
+		return -1;
+
+	if (is_valid_id(be32toh(gard->record_id)))
+		(*(int *)priv)++;
+
+	return 0;
+}
+
+static int count_valid_records(struct gard_ctx *ctx)
+{
+	int rc, count = 0;
+
+	rc = do_iterate(ctx, &count_valid_records_i, &count);
+	if (rc)
+		return 0;
+
+	return count;
+}
+
 static int do_list_i(struct gard_ctx *ctx, int pos, struct gard_record *gard, void *priv)
 {
 	if (!gard)
 		return -1;
 
-	printf("| %08x | %08x | %-15s |\n", be32toh(gard->record_id),
-	       be32toh(gard->errlog_eid), path_type_to_str(gard->target_id.type_size >> PATH_TYPE_SHIFT));
+	if (is_valid_id(be32toh(gard->record_id)))
+		printf("| %08x | %08x | %-15s |\n", be32toh(gard->record_id), be32toh(gard->errlog_eid),
+		       path_type_to_str(gard->target_id.type_size >> PATH_TYPE_SHIFT));
 
 	return 0;
 }
@@ -420,7 +451,7 @@ static int do_list(struct gard_ctx *ctx, int argc, char **argv)
 	int rc;
 
 	/* No entries */
-	if (get_largest_pos(ctx) == -1) {
+	if (count_valid_records(ctx) == 0) {
 		printf("No GARD entries to display\n");
 		rc = 0;
 	} else {
@@ -479,73 +510,83 @@ static int do_show(struct gard_ctx *ctx, int argc, char **argv)
 
 static int do_clear_i(struct gard_ctx *ctx, int pos, struct gard_record *gard, void *priv)
 {
-	int largest = 0, rc = 0;
+	int largest, rc = 0;
 	char *buf;
 	struct gard_record null_gard;
 
-	if (!gard || !ctx)
+	if (!gard || !ctx || !priv)
 		return -1;
 
-	memset(&null_gard, INT_MAX, sizeof(null_gard));
+	/* Not this one */
+	if (be32toh(gard->record_id) != *(uint32_t *)priv)
+		return 0;
 
-	if (!priv) {
-		if (pos != 0)
-			/* We erased everything on the first iteration, don't bother */
-			return 0;
+	memset(&null_gard, 0xFF, sizeof(null_gard));
 
-		printf("Erasing the entire gard partition...");
-		rc = blocklevel_erase(ctx->bl, ctx->gard_data_pos, ctx->gard_data_len);
+	largest = get_largest_pos(ctx);
+
+	printf("Clearing gard record 0x%08x...", be32toh(gard->record_id));
+
+	if (largest < 0 || pos > largest) {
+		/* Something went horribly wrong */
+		fprintf(stderr, "largest index out of range %d\n", largest);
+		return -1;
+	}
+
+	if (pos < largest) {
+		/* We're not clearing the last record, shift all the records up */
+		int buf_len = ((largest - pos) * sizeof(struct gard_record));
+		int buf_pos = ctx->gard_data_pos + ((pos + 1) * sizeof_gard(ctx));
+		buf = malloc(buf_len);
+		if (!buf)
+			return -ENOMEM;
+
+		rc = blocklevel_read(ctx->bl, buf_pos, buf, buf_len);
 		if (rc) {
-			fprintf(stderr, "\nCouldn't erase flash partition at 0x%08x for size %u\n",
-					ctx->gard_data_pos, ctx->gard_data_len);
+			free(buf);
+			fprintf(stderr, "Couldn't read from flash at 0x%08x for len 0x%08x\n", buf_pos, buf_len);
 			return rc;
 		}
-		printf("done\n");
 
-		ctx->gard_data_len = sizeof_gard(ctx);
-	} else if (be32toh(gard->record_id) == *(uint32_t *)priv) {
-		largest = get_largest_pos(ctx);
-		if (largest < 0 || pos > largest) {
-			/* Something went horribly wrong */
-			fprintf(stderr, "largest index out of range %d\n", largest);
-			return -1;
+		rc = blocklevel_smart_write(ctx->bl, buf_pos - sizeof_gard(ctx), buf, buf_len);
+		free(buf);
+		if (rc) {
+			fprintf(stderr, "Couldn't write to flash at 0x%08lx for len 0x%08x\n",
+			        buf_pos - sizeof_gard(ctx), buf_len);
+			return rc;
 		}
-
-		if (pos < largest) {
-			/* We're not clearing the last record, shift all the records up */
-			int buf_len = ((largest - pos) * sizeof(struct gard_record));
-			int buf_pos = ctx->gard_data_pos + ((pos + 1) * sizeof_gard(ctx));
-			buf = malloc(buf_len);
-			if (!buf)
-				return -ENOMEM;
-
-			rc = blocklevel_read(ctx->bl, buf_pos, buf, buf_len);
-			if (rc) {
-				free(buf);
-				fprintf(stderr, "Couldn't read from flash at 0x%08x for len 0x%08x\n", buf_pos, buf_len);
-				return rc;
-			}
-
-			rc = blocklevel_smart_write(ctx->bl, buf_pos - sizeof_gard(ctx), buf, buf_len);
-			free(buf);
-			if (rc) {
-				fprintf(stderr, "Couldn't write to flash at 0x%08lx for len 0x%08x\n",
-				        buf_pos - sizeof_gard(ctx), buf_len);
-				return rc;
-			}
-		}
-
-		ctx->gard_data_len -= sizeof_gard(ctx);
-		printf("Cleared gard record with id ID 0x%08x\n", be32toh(gard->record_id));
 	}
 
 	/* Now wipe the last record */
 	rc = blocklevel_smart_write(ctx->bl, ctx->gard_data_pos + (largest * sizeof_gard(ctx)),
-		                            &null_gard, sizeof(null_gard));
+	                            &null_gard, sizeof(null_gard));
+	printf("done\n");
 
 	return rc;
 }
 
+static int reset_partition(struct gard_ctx *ctx)
+{
+	int i, rc;
+	struct gard_record gard;
+	memset(&gard, 0xFF, sizeof(gard));
+
+	rc = blocklevel_erase(ctx->bl, ctx->gard_data_pos, ctx->gard_data_len);
+	if (rc) {
+		fprintf(stderr, "Couldn't erase the gard partition. Bailing out\n");
+		return rc;
+	}
+	for (i = 0; i + sizeof_gard(ctx) < ctx->gard_data_len; i += sizeof_gard(ctx)) {
+		rc = blocklevel_write(ctx->bl, ctx->gard_data_pos + i, &gard, sizeof(gard));
+		if (rc) {
+			fprintf(stderr, "Couldn't reset the entire gard partition. Bailing out\n");
+			return rc;
+		}
+	}
+
+	return 0;
+}
+
 static int do_clear(struct gard_ctx *ctx, int argc, char **argv)
 {
 	int rc;
@@ -557,7 +598,10 @@ static int do_clear(struct gard_ctx *ctx, int argc, char **argv)
 	}
 
 	if (strncmp(argv[1], "all", strlen("all")) == 0) {
-		rc = do_iterate(ctx, do_clear_i, NULL);
+		printf("Clearing the entire gard partition...");
+		fflush(stdout);
+		rc = reset_partition(ctx);
+		printf("done\n");
 	} else {
 		id = strtoul(argv[1], NULL, 16);
 		rc = do_iterate(ctx, do_clear_i, &id);
@@ -566,6 +610,46 @@ static int do_clear(struct gard_ctx *ctx, int argc, char **argv)
 	return rc;
 }
 
+int check_gard_partition(struct gard_ctx *ctx)
+{
+	int rc;
+	struct gard_record gard;
+	char msg[2];
+
+	if (ctx->gard_data_len == 0 || ctx->gard_data_len % sizeof(struct gard_record) != 0)
+		/* Just warn for now */
+		fprintf(stderr, "The %s partition doesn't appear to be an exact multiple of"
+				"gard records in size: %lu vs %u (or partition is zero in length)\n",
+				FLASH_GARD_PART, sizeof(struct gard_record), ctx->gard_data_len);
+
+	/*
+	 * Attempt to read the first record, nothing can really operate if the
+	 * first record is dead. There (currently) isn't a way to validate more
+	 * than ECC correctness.
+	 */
+	rc = blocklevel_read(ctx->bl, ctx->gard_data_pos, &gard, sizeof(gard));
+	if (rc == FLASH_ERR_ECC_INVALID) {
+		fprintf(stderr, "The data at the GUARD partition does not appear to be valid gard data\n");
+		fprintf(stderr, "Clear the entire GUARD partition? [y/N]\n");
+		if (fgets(msg, sizeof(msg), stdin) == NULL) {
+			fprintf(stderr, "Couldn't read from standard input\n");
+			return -1;
+		}
+		if (msg[0] == 'y') {
+			rc = reset_partition(ctx);
+			if (rc) {
+				fprintf(stderr, "Couldn't reset the GUARD partition. Bailing out\n");
+				return rc;
+			}
+		}
+		/*
+		 * else leave rc as is so that the main bails out, not going to be
+		 * able to do sensible anyway
+		 */
+	}
+	return rc;
+}
+
 __attribute__ ((unused))
 static int do_nop(struct gard_ctx *ctx, int argc, char **argv)
 {
@@ -620,7 +704,7 @@ int main(int argc, char **argv)
 	const char *fdt_flash_path = FDT_ACTIVE_FLASH_PATH;
 	char *filename = NULL;
 	struct gard_ctx _ctx, *ctx;
-	int i, rc;
+	int rc, i = 0;
 	bool part = 0;
 	bool ecc = 0;
 
@@ -718,11 +802,11 @@ int main(int argc, char **argv)
 		ctx->gard_data_len = ctx->f_size;
 	}
 
-	if (ctx->gard_data_len == 0 || ctx->gard_data_len % sizeof(struct gard_record) != 0)
-		/* Just warn for now */
-		fprintf(stderr, "The %s partition doesn't appear to be an exact multiple of"
-				"gard records in size: %lu vs %u (or partition is zero in length)\n",
-				FLASH_GARD_PART, sizeof(struct gard_record), ctx->gard_data_len);
+	rc = check_gard_partition(ctx);
+	if (rc) {
+		fprintf(stderr, "Does not appear to be sane gard data\n");
+		goto out;
+	}
 
 	for (i = 0; i < ARRAY_SIZE(actions); i++) {
 		if (!strcmp(actions[i].name, action)) {
