@@ -25,6 +25,7 @@
 #include <p5ioc2.h>
 #include <p7ioc.h>
 #include <vpd.h>
+#include <inttypes.h>
 
 #include "hdata.h"
 
@@ -32,11 +33,11 @@ static void io_add_common(struct dt_node *hn, const struct cechub_io_hub *hub)
 {
 	dt_add_property_cells(hn, "#address-cells", 2);
 	dt_add_property_cells(hn, "#size-cells", 2);
-	dt_add_property_cells(hn, "ibm,buid-ext", hub->buid_ext);
+	dt_add_property_cells(hn, "ibm,buid-ext", be32_to_cpu(hub->buid_ext));
 	dt_add_property_cells(hn, "ibm,chip-id",
-			      pcid_to_chip_id(hub->proc_chip_id));
-	dt_add_property_cells(hn, "ibm,gx-index", hub->gx_index);
-	dt_add_property_cells(hn, "revision", hub->ec_level);
+			      pcid_to_chip_id(be32_to_cpu(hub->proc_chip_id)));
+	dt_add_property_cells(hn, "ibm,gx-index", be32_to_cpu(hub->gx_index));
+	dt_add_property_cells(hn, "revision", be32_to_cpu(hub->ec_level));
 
 	/* Instead of exposing the GX BARs as separate ranges as we *should*
 	 * do in an ideal world, we just create a pass-through ranges and
@@ -111,7 +112,7 @@ static void io_get_loc_code(const void *sp_iohubs, struct dt_node *hn, const cha
 		memset(loc_code, 0, sizeof(loc_code));
 
 		/* Find LOC Code from SLCA Index */
-		slca_loc_code = slca_get_loc_code_index(fru_id->slca_index);
+		slca_loc_code = slca_get_loc_code_index(be16_to_cpu(fru_id->slca_index));
 		if (slca_loc_code) {
 			strncpy(loc_code, slca_loc_code, LOC_CODE_SIZE);
 			if (!dt_has_node_property(hn, prop_name, NULL)) {
@@ -142,15 +143,15 @@ static struct dt_node *io_add_p5ioc2(const struct cechub_io_hub *hub,
 	prlog(PR_DEBUG, "    GX#%d BUID_Ext = 0x%x\n",
 	      be32_to_cpu(hub->gx_index),
 	      be32_to_cpu(hub->buid_ext));
-	prlog(PR_DEBUG, "    GX BAR 0 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 0 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar0));
-	prlog(PR_DEBUG, "    GX BAR 1 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 1 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar1));
-	prlog(PR_DEBUG, "    GX BAR 2 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 2 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar2));
-	prlog(PR_DEBUG, "    GX BAR 3 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 3 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar3));
-	prlog(PR_DEBUG, "    GX BAR 4 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 4 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar4));
 
 	/* We assume SBAR == GX0 + some hard coded offset */
@@ -194,15 +195,15 @@ static struct dt_node *io_add_p7ioc(const struct cechub_io_hub *hub,
 	prlog(PR_DEBUG, "    GX#%d BUID_Ext = 0x%x\n",
 	      be32_to_cpu(hub->gx_index),
 	      be32_to_cpu(hub->buid_ext));
-	prlog(PR_DEBUG, "    GX BAR 0 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 0 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar0));
-	prlog(PR_DEBUG, "    GX BAR 1 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 1 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar1));
-	prlog(PR_DEBUG, "    GX BAR 2 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 2 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar2));
-	prlog(PR_DEBUG, "    GX BAR 3 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 3 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar3));
-	prlog(PR_DEBUG, "    GX BAR 4 = 0x%016llx\n",
+	prlog(PR_DEBUG, "    GX BAR 4 = 0x%016"PRIx64"\n",
 	      be64_to_cpu(hub->gx_ctrl_bar4));
 
 	/* We only know about memory map 1 */
@@ -634,7 +635,7 @@ static void io_parse_fru(const void *sp_iohubs)
 		prlog(PR_DEBUG, "CEC:   PChip: %d HUB ID: %04x [EC=0x%x]"
 		      " Hub#=%d)\n",
 		      be32_to_cpu(hub->proc_chip_id), hub_id,
-		      be32_to_cpu(hub->ec_level), be32_to_cpu(hub->hub_num));
+		      be32_to_cpu(hub->ec_level), be16_to_cpu(hub->hub_num));
 
 		switch(hub_id) {
 		case CECHUB_HUB_P7IOC:
@@ -705,7 +706,7 @@ void io_parse(void)
 			prerror("CEC: IO-HUB FRU %d, bad ID data\n", i);
 			continue;
 		}
-		type = fru_id_data->card_type;
+		type = be32_to_cpu(fru_id_data->card_type);
 
 		prlog(PR_INFO, "CEC: HUB FRU %d is %s\n",
 		      i, type > 4 ? "Unknown" : typestr[type]);
