@@ -23,6 +23,7 @@
 #include <chip.h>
 #include <fsp-mdst-table.h>
 #include <fsp-attn.h>
+#include <fsp-leds.h>
 
 #include "hdata.h"
 #include "hostservices.h"
@@ -736,6 +737,7 @@ static void add_iplparams_sys_params(const void *iplp, struct dt_node *node)
 static void add_iplparams_ipl_params(const void *iplp, struct dt_node *node)
 {
 	const struct iplparams_iplparams *p;
+	struct dt_node *led_node;
 
 	p = HDIF_get_idata(iplp, IPLPARAMS_IPLPARAMS, NULL);
 	if (!CHECK_SPPTR(p)) {
@@ -763,6 +765,17 @@ static void add_iplparams_ipl_params(const void *iplp, struct dt_node *node)
 	dt_add_property_cells(node, "os-ipl-mode", p->os_ipl_mode);
 	dt_add_property_strings(node, "cec-major-type",
 				p->cec_ipl_maj_type ? "hot" : "cold");
+
+	/* Add LED type info under '/ibm,opal/led' node */
+	led_node = dt_find_by_path(opal_node, DT_PROPERTY_LED_NODE);
+	assert(led_node);
+
+	if (p->other_attrib & IPLPARAMS_OATRR_LIGHT_PATH)
+		dt_add_property_strings(led_node, DT_PROPERTY_LED_MODE,
+					LED_MODE_LIGHT_PATH);
+	else
+		dt_add_property_strings(led_node, DT_PROPERTY_LED_MODE,
+					LED_MODE_GUIDING_LIGHT);
 }
 
 static void add_iplparams_serials(const void *iplp, struct dt_node *node)
@@ -906,12 +919,29 @@ uint32_t pcid_to_chip_id(uint32_t proc_chip_id)
 	return (uint32_t)-1;
 }
 
+/* Create '/ibm,opal/led' node */
+static void dt_init_led_node(void)
+{
+	struct dt_node *led_node;
+
+	/* Create /ibm,opal node, if its not created already */
+	if (!opal_node) {
+		opal_node = dt_new(dt_root, "ibm,opal");
+		assert(opal_node);
+	}
+
+	/* Crete LED parent node */
+	led_node = dt_new(opal_node, DT_PROPERTY_LED_NODE);
+	assert(led_node);
+}
+
 static void dt_init_vpd_node(void)
 {
 	struct dt_node *dt_vpd;
 
 	dt_vpd = dt_new(dt_root, "vpd");
 	assert(dt_vpd);
+	dt_add_property_string(dt_vpd, "compatible", "ibm,opal-v3-vpd");
 }
 
 static void hostservices_parse(void)
@@ -962,6 +992,9 @@ void parse_hdat(bool is_opal, uint32_t master_cpu)
 	/* Create /vpd node */
 	dt_init_vpd_node();
 
+	/* Create /ibm,opal/led node */
+	dt_init_led_node();
+
 	/* Parse SPPACA and/or PCIA */
 	if (!pcia_parse())
 		paca_parse();
@@ -994,5 +1027,8 @@ void parse_hdat(bool is_opal, uint32_t master_cpu)
 	/* Host services information. */
  	hostservices_parse();
 
+	/* Parse System Attention Indicator inforamtion */
+	slca_dt_add_sai_node();
+
 	prlog(PR_INFO, "Parsing HDAT...done\n");
 }
