@@ -57,6 +57,7 @@ static enum led_support_state led_support = LED_STATE_ABSENT;
  *
  */
 static void *led_buffer;
+static u8 *loc_code_list_buffer = NULL;
 
 /* Maintain list of all LEDs
  *
@@ -175,7 +176,7 @@ static void compute_encl_status_cec(struct fsp_led_data *encl_led)
 		if (!strstr(led->loc_code, encl_led->loc_code))
 			continue;
 
-		/* Dont count the enclsure LED itself */
+		/* Don't count the enclsure LED itself */
 		if (!strcmp(led->loc_code, encl_led->loc_code))
 			continue;
 
@@ -350,7 +351,7 @@ static bool sai_update_notification(struct fsp_msg *msg)
 
 /*
  * Update both the local LED lists to reflect upon led state changes
- * occured with the recent SPCN command. Subsequent LED requests will
+ * occurred with the recent SPCN command. Subsequent LED requests will
  * be served with these updates changed to the list.
  */
 static void update_led_list(char *loc_code, u32 led_state, u32 excl_bit)
@@ -751,7 +752,7 @@ static u32 fsp_push_data_to_tce(struct fsp_led_data *led, u8 *out_data,
 
 	/*
 	 * Check for outbound buffer overflow. If there are still
-	 * more LEDs to be sent across to FSP, dont send, ignore.
+	 * more LEDs to be sent across to FSP, don't send, ignore.
 	 */
 	if ((total_size + lcode.size) > PSI_DMA_LOC_COD_BUF_SZ)
 		return 0;
@@ -777,8 +778,13 @@ static void fsp_ret_loc_code_list(u16 req_type, char *loc_code)
 	u32 bytes_sent = 0, total_size = 0;
 	u16 header_size = 0, flags = 0;
 
+	if (loc_code_list_buffer == NULL) {
+		prerror("No loc_code_list_buffer\n");
+		return;
+	}
+
 	/* Init the addresses */
-	data = (u8 *) PSI_DMA_LOC_COD_BUF;
+	data = loc_code_list_buffer;
 	out_data = NULL;
 
 	/* Unmapping through FSP_CMD_RET_LOC_BUFFER command */
@@ -1582,7 +1588,7 @@ static void fsp_process_leds_data(u16 len)
 	 * Process the entire captured data from the last command
 	 *
 	 * TCE mapped 'led_buffer' contains the fsp_led_data structure
-	 * one after the other till the total lenght 'len'.
+	 * one after the other till the total length 'len'.
 	 *
 	 */
 	buf = led_buffer;
@@ -1875,6 +1881,11 @@ void fsp_led_init(void)
 	list_head_init(&spcn_cmdq);
 
 	fsp_leds_query_spcn();
+
+	loc_code_list_buffer = memalign(TCE_PSIZE, PSI_DMA_LOC_COD_BUF_SZ);
+	if (loc_code_list_buffer == NULL)
+		prerror("ERROR: Unable to allocate loc_code_list_buffer!\n");
+
 	prlog(PR_TRACE, "Init completed\n");
 
 	/* Get System attention indicator state */
