@@ -50,15 +50,15 @@
 DEFINE_LOG_ENTRY(OPAL_RC_SENSOR_INIT, OPAL_PLATFORM_ERR_EVT, OPAL_SENSOR,
 			OPAL_MISC_SUBSYSTEM,
 			OPAL_PREDICTIVE_ERR_FAULT_RECTIFY_REBOOT,
-			OPAL_NA, NULL);
+			OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_SENSOR_READ, OPAL_PLATFORM_ERR_EVT, OPAL_SENSOR,
 			OPAL_MISC_SUBSYSTEM, OPAL_INFO,
-			OPAL_NA, NULL);
+			OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_SENSOR_ASYNC_COMPLETE, OPAL_PLATFORM_ERR_EVT,
 			OPAL_SENSOR, OPAL_MISC_SUBSYSTEM, OPAL_INFO,
-			OPAL_NA, NULL);
+			OPAL_NA);
 
 /* FSP response status codes */
 enum {
