@@ -80,6 +80,16 @@ struct fsp {
 	struct fsp_iopath	iopath[FSP_MAX_IOPATH];
 };
 
+enum ipl_state {
+	ipl_initial		= 0x00000000,
+	ipl_opl_sent		= 0x00000001,
+	ipl_got_continue	= 0x00000002,
+	ipl_got_new_role	= 0x00000004,
+	ipl_got_caps		= 0x00000008,
+	ipl_got_fsp_functional	= 0x00000010
+};
+static enum ipl_state ipl_state = ipl_initial;
+
 static struct fsp *first_fsp;
 static struct fsp *active_fsp;
 static u16 fsp_curseq = 0x8000;
@@ -109,7 +119,7 @@ static u64 fsp_hir_timeout;
 
 /*
  * We keep track on last logged values for some things to print only on
- * value changes, but also to releive pressure on the tracer which
+ * value changes, but also to relieve pressure on the tracer which
  * doesn't do a very good job at detecting repeats when called from
  * many different CPUs
  */
@@ -559,6 +569,28 @@ static void fsp_start_rr(struct fsp *fsp)
 	 */
 }
 
+/*
+ * Called on normal/quick shutdown to give up the PSI link
+ */
+void fsp_reset_links(void)
+{
+	struct fsp *fsp = fsp_get_active();
+	struct fsp_iopath *iop;
+
+	if (!fsp)
+		return;
+
+	/* Already in one of the error states? */
+	if (fsp_in_hir(fsp) || fsp_in_reset(fsp))
+		return;
+
+	iop = &fsp->iopath[fsp->active_iopath];
+	prlog(PR_NOTICE, "FSP #%d: Host initiated shutdown."
+			" Giving up the PSI link\n", fsp->index);
+	psi_disable_link(iop->psi);
+	return;
+}
+
 static void fsp_trace_event(struct fsp *fsp, u32 evt,
 			    u32 data0, u32 data1, u32 data2, u32 data3)
 {
@@ -656,9 +688,23 @@ static void fsp_handle_errors(struct fsp *fsp)
 		if (fsp->state == fsp_mbx_rr)
 			return;
 
-		prlog(PR_NOTICE, "FSP #%d: FSP in Reset."
-		      " Waiting for PSI interrupt\n",
-		      fsp->index);
+		if (fsp_dpo_pending) {
+			/*
+			 * If we are about to process a reset when DPO
+			 * is pending, its possible that the host has
+			 * gone down, and OPAL is on its way down and
+			 * hence will not see the subsequent PSI interrupt.
+			 * So, just give up the link here.
+			 */
+			prlog(PR_NOTICE, "FSP #%d: FSP reset with DPO pending."
+					" Giving up PSI link\n",
+					fsp->index);
+			psi_disable_link(psi);
+		} else {
+			prlog(PR_NOTICE, "FSP #%d: FSP in Reset."
+				" Waiting for PSI interrupt\n",
+				fsp->index);
+		}
 		fsp_start_rr(fsp);
 	}
 
@@ -1914,7 +1960,9 @@ static void fsp_create_fsp(struct dt_node *fsp_node)
 
 	fsp_init_links(fsp_node);
 	fsp_update_links_states(fsp);
-	psi_enable_fsp_interrupt(fsp->iopath[fsp->active_iopath].psi);
+
+	if (fsp->active_iopath >= 0)
+		psi_enable_fsp_interrupt(fsp->iopath[fsp->active_iopath].psi);
 }
 
 static void fsp_opal_poll(void *data __unused)
