@@ -125,15 +125,64 @@ struct oem_sel {
 	uint8_t data[4];
 };
 
-/* As far as I can tell the size of PEL record is unbounded (due to
- * the possible presence of the user defined section). We chose this
- * size because it's what hostboot also uses and most of the OPAL logs
- * are few hundred bytes.
- */
-#define MAX_PEL_SIZE 0x800
-
 #define ESEL_HDR_SIZE 7
 
+/* Used for sending PANIC events like abort() path */
+struct ipmi_sel_panic_msg {
+	bool		busy;
+	struct ipmi_msg	*msg;
+	struct lock	lock;
+};
+static struct ipmi_sel_panic_msg ipmi_sel_panic_msg;
+
+/* Forward declaration */
+static void ipmi_elog_poll(struct ipmi_msg *msg);
+
+/*
+ * Allocate IPMI message
+ *  For normal event, allocate memory using ipmi_mkmsg and for PANIC
+ *  event, use pre-allocated buffer.
+ */
+static struct ipmi_msg *ipmi_sel_alloc_msg(struct errorlog *elog_buf)
+{
+	struct ipmi_msg *msg = NULL;
+
+	if (elog_buf->event_severity == OPAL_ERROR_PANIC) {
+		/* Called before initialization completes */
+		if (ipmi_sel_panic_msg.msg == NULL)
+			return NULL;
+
+		if (ipmi_sel_panic_msg.busy == true)
+			return NULL;
+
+		lock(&ipmi_sel_panic_msg.lock);
+		msg = ipmi_sel_panic_msg.msg;
+		ipmi_sel_panic_msg.busy = true;
+		unlock(&ipmi_sel_panic_msg.lock);
+
+		ipmi_init_msg(msg, IPMI_DEFAULT_INTERFACE,
+			      IPMI_RESERVE_SEL, ipmi_elog_poll,
+			      elog_buf, IPMI_MAX_REQ_SIZE, 2);
+	} else {
+		msg = ipmi_mkmsg(IPMI_DEFAULT_INTERFACE, IPMI_RESERVE_SEL,
+				 ipmi_elog_poll, elog_buf,
+				 NULL, IPMI_MAX_REQ_SIZE, 2);
+	}
+
+	return msg;
+}
+
+static void ipmi_sel_free_msg(struct ipmi_msg *msg)
+{
+	if (msg == ipmi_sel_panic_msg.msg) {
+		lock(&ipmi_sel_panic_msg.lock);
+		ipmi_sel_panic_msg.busy = false;
+		unlock(&ipmi_sel_panic_msg.lock);
+	} else {
+		ipmi_free_msg(msg);
+	}
+	msg = NULL;
+}
 
 /* Initialize eSEL record */
 static void ipmi_init_esel_record(void)
@@ -199,7 +248,7 @@ static void ipmi_elog_error(struct ipmi_msg *msg)
 		ipmi_queue_msg(msg);
 	else {
 		opal_elog_complete(msg->user_data, false);
-		ipmi_free_msg(msg);
+		ipmi_sel_free_msg(msg);
 	}
 }
 
@@ -208,7 +257,7 @@ static void ipmi_log_sel_event_error(struct ipmi_msg *msg)
 	if (msg->cc != IPMI_CC_NO_ERROR)
 		prlog(PR_INFO, "SEL: Failed to log SEL event\n");
 
-	ipmi_free_msg(msg);
+	ipmi_sel_free_msg(msg);
 }
 
 static void ipmi_log_sel_event_complete(struct ipmi_msg *msg)
@@ -216,27 +265,26 @@ static void ipmi_log_sel_event_complete(struct ipmi_msg *msg)
 	prlog(PR_INFO, "SEL: New event logged [ID : %x%x]\n",
 	      msg->data[1], msg->data[0]);
 
-	ipmi_free_msg(msg);
+	ipmi_sel_free_msg(msg);
 }
 
 /* Log SEL event with eSEL record ID */
-static void ipmi_log_sel_event(uint8_t event_severity, uint16_t esel_record_id)
+static void ipmi_log_sel_event(struct ipmi_msg *msg,
+			       uint8_t event_severity, uint16_t esel_record_id)
 {
-	struct ipmi_msg *msg;
-
 	/* Fill required SEL event fields */
 	ipmi_update_sel_record(event_severity, esel_record_id);
 
-	msg = ipmi_mkmsg(IPMI_DEFAULT_INTERFACE, IPMI_ADD_SEL_EVENT,
-			 ipmi_log_sel_event_complete, NULL,
-			 &sel_record, sizeof(struct sel_record), 2);
-	if (!msg) {
-		prlog(PR_ERR, "SEL: Failed to allocated IPMI message\n");
-		return;
-	}
+	/* Fill IPMI message */
+	ipmi_init_msg(msg, IPMI_DEFAULT_INTERFACE, IPMI_ADD_SEL_EVENT,
+		      ipmi_log_sel_event_complete, NULL,
+		      sizeof(struct sel_record), 2);
+
+	/* Copy SEL data */
+	memcpy(msg->data, &sel_record, sizeof(struct sel_record));
 
 	msg->error = ipmi_log_sel_event_error;
-	ipmi_queue_msg(msg);
+	ipmi_queue_msg_head(msg);
 }
 
 /* Goes through the required steps to add a complete eSEL:
@@ -261,7 +309,7 @@ static void ipmi_log_sel_event(uint8_t event_severity, uint16_t esel_record_id)
 static void ipmi_elog_poll(struct ipmi_msg *msg)
 {
 	static bool first = false;
-	static char pel_buf[MAX_PEL_SIZE];
+	static char pel_buf[IPMI_MAX_PEL_SIZE];
 	static size_t pel_size;
 	static size_t esel_size;
 	static int esel_index = 0;
@@ -283,11 +331,12 @@ static void ipmi_elog_poll(struct ipmi_msg *msg)
 			 * sending the message. */
 			prerror("Invalid reservation id");
 			opal_elog_complete(elog_buf, false);
-			ipmi_free_msg(msg);
+			ipmi_sel_free_msg(msg);
 			return;
 		}
 
-		pel_size = create_pel_log(elog_buf, pel_buf, MAX_PEL_SIZE);
+		pel_size = create_pel_log(elog_buf,
+					  pel_buf, IPMI_MAX_PEL_SIZE);
 		esel_size = pel_size + sizeof(struct sel_record);
 		esel_index = 0;
 		record_id = 0;
@@ -304,15 +353,14 @@ static void ipmi_elog_poll(struct ipmi_msg *msg)
 		reservation_id = 0;
 		esel_index = 0;
 
-		/* Log SEL event */
-		ipmi_log_sel_event(elog_buf->event_severity, record_id);
+		/* Log SEL event and free ipmi message */
+		ipmi_log_sel_event(msg, elog_buf->event_severity, record_id);
 
 		opal_elog_complete(elog_buf, true);
-		ipmi_free_msg(msg);
 		return;
 	}
 
-	if ((esel_size - esel_index) < (IPMI_MAX_REQ_SIZE - ESEL_HDR_SIZE)) {
+	if ((esel_size - esel_index) <= (IPMI_MAX_REQ_SIZE - ESEL_HDR_SIZE)) {
 		/* Last data to send */
 		msg->data[6] = 1;
 		req_size = esel_size - esel_index + ESEL_HDR_SIZE;
@@ -362,8 +410,7 @@ int ipmi_elog_commit(struct errorlog *elog_buf)
 	/* We pass a large request size in to mkmsg so that we have a
 	 * large enough allocation to reuse the message to pass the
 	 * PEL data via a series of partial add commands.  */
-	msg = ipmi_mkmsg(IPMI_DEFAULT_INTERFACE, IPMI_RESERVE_SEL, ipmi_elog_poll,
-			 elog_buf, NULL, IPMI_MAX_REQ_SIZE, 2);
+	msg = ipmi_sel_alloc_msg(elog_buf);
 	if (!msg) {
 		opal_elog_complete(elog_buf, false);
 		return OPAL_RESOURCE;
@@ -410,12 +457,24 @@ static void sel_power(uint8_t power)
 {
 	switch (power) {
 	case SOFT_OFF:
-		prlog(PR_NOTICE, "soft shutdown requested\n");
-		opal_queue_msg(OPAL_MSG_SHUTDOWN, NULL, NULL, SOFT_OFF);
+		prlog(PR_NOTICE, "Soft shutdown requested\n");
+		if (!(debug_descriptor.state_flags & OPAL_BOOT_COMPLETE) &&
+		    platform.cec_power_down) {
+			prlog(PR_NOTICE, "Host not up, shutting down now\n");
+			platform.cec_power_down(IPMI_CHASSIS_PWR_DOWN);
+		} else {
+			opal_queue_msg(OPAL_MSG_SHUTDOWN, NULL, NULL, SOFT_OFF);
+		}
 		break;
 	case SOFT_REBOOT:
-		prlog(PR_NOTICE, "soft reboot rqeuested\n");
-		opal_queue_msg(OPAL_MSG_SHUTDOWN, NULL, NULL, SOFT_REBOOT);
+		prlog(PR_NOTICE, "Soft reboot requested\n");
+		if (!(debug_descriptor.state_flags & OPAL_BOOT_COMPLETE) &&
+		    platform.cec_reboot) {
+			prlog(PR_NOTICE, "Host not up, rebooting now\n");
+			platform.cec_reboot();
+		} else {
+			opal_queue_msg(OPAL_MSG_SHUTDOWN, NULL, NULL, SOFT_REBOOT);
+		}
 		break;
 	default:
 		prlog(PR_WARNING, "requested bad power state: %02x\n",
@@ -488,3 +547,11 @@ void ipmi_parse_sel(struct ipmi_msg *msg)
 		      sel.cmd);
 	}
 }
+
+void ipmi_sel_init(void)
+{
+	memset(&ipmi_sel_panic_msg, 0, sizeof(struct ipmi_sel_panic_msg));
+	ipmi_sel_panic_msg.msg = ipmi_mkmsg(IPMI_DEFAULT_INTERFACE,
+					    IPMI_RESERVE_SEL, ipmi_elog_poll,
+					    NULL, NULL, IPMI_MAX_REQ_SIZE, 2);
+}
