@@ -26,6 +26,7 @@
 #include <errorlog.h>
 #include <opal-api.h>
 #include <opal-msg.h>
+#include <timer.h>
 
 /* OCC Communication Area for PStates */
 
@@ -311,6 +312,14 @@ static bool cpu_pstates_prepare_core(struct proc_chip *chip, struct cpu_thread *
 	return true;
 }
 
+static bool occ_opal_msg_outstanding = false;
+static void occ_msg_consumed(void *data __unused)
+{
+	lock(&occ_lock);
+	occ_opal_msg_outstanding = false;
+	unlock(&occ_lock);
+}
+
 static void occ_throttle_poll(void *data __unused)
 {
 	struct proc_chip *chip;
@@ -344,6 +353,8 @@ static void occ_throttle_poll(void *data __unused)
 				occ_reset = false;
 		}
 	} else {
+		if (occ_opal_msg_outstanding)
+			goto done;
 		for_each_chip(chip) {
 			occ_data = chip_occ_data(chip);
 			if ((occ_data->valid == 1) &&
@@ -352,13 +363,18 @@ static void occ_throttle_poll(void *data __unused)
 				occ_msg.type = OCC_THROTTLE;
 				occ_msg.chip = chip->id;
 				occ_msg.throttle_status = occ_data->throttle;
-				rc = _opal_queue_msg(OPAL_MSG_OCC, NULL, NULL,
+				rc = _opal_queue_msg(OPAL_MSG_OCC, NULL,
+						     occ_msg_consumed,
 						     3, (uint64_t *)&occ_msg);
-				if (!rc)
+				if (!rc) {
 					chip->throttle = occ_data->throttle;
+					occ_opal_msg_outstanding = true;
+					break;
+				}
 			}
 		}
 	}
+done:
 	unlock(&occ_lock);
 }
 
@@ -488,12 +504,12 @@ static void __occ_do_load(u8 scope, u32 dbob_id __unused, u32 seq_id)
 
 void occ_poke_load_queue(void)
 {
-	struct occ_load_req *occ_req;
+	struct occ_load_req *occ_req, *next;
 
 	if (list_empty(&occ_load_req_list))
 		return;
 
-	list_for_each(&occ_load_req_list, occ_req, link) {
+	list_for_each_safe(&occ_load_req_list, occ_req, next, link) {
 		__occ_do_load(occ_req->scope, occ_req->dbob_id,
 				occ_req->seq_id);
 		list_del(&occ_req->link);
@@ -715,9 +731,11 @@ static struct fsp_client fsp_occ_client = {
 #define OCB_OCI_OCCMISC_OR	0x6a022
 #define OCB_OCI_OCIMISC_IRQ		PPC_BIT(0)
 #define OCB_OCI_OCIMISC_IRQ_TMGT	PPC_BIT(1)
+#define OCB_OCI_OCIMISC_IRQ_SLW_TMR	PPC_BIT(14)
 #define OCB_OCI_OCIMISC_IRQ_OPAL_DUMMY	PPC_BIT(15)
 #define OCB_OCI_OCIMISC_MASK		(OCB_OCI_OCIMISC_IRQ_TMGT | \
-					 OCB_OCI_OCIMISC_IRQ_OPAL_DUMMY )
+					 OCB_OCI_OCIMISC_IRQ_OPAL_DUMMY | \
+					 OCB_OCI_OCIMISC_IRQ_SLW_TMR)
 
 void occ_send_dummy_interrupt(void)
 {
@@ -765,6 +783,8 @@ void occ_interrupt(uint32_t chip_id)
 	/* Dispatch */
 	if (ireg & OCB_OCI_OCIMISC_IRQ_TMGT)
 		prd_tmgt_interrupt(chip_id);
+	if (ireg & OCB_OCI_OCIMISC_IRQ_SLW_TMR)
+		check_timers(true);
 
 	/* We may have masked-out OCB_OCI_OCIMISC_IRQ in the previous
 	 * OCCMISC_AND write. Check if there are any new source bits set,
