@@ -29,18 +29,6 @@
 			   SPR_HMER_XSCOM_DONE | \
 			   SPR_HMER_XSCOM_STATUS))
 
-#define XSCOM_ADDR_IND_FLAG		PPC_BIT(0)
-#define XSCOM_ADDR_IND_ADDR		PPC_BITMASK(12,31)
-#define XSCOM_ADDR_IND_DATA		PPC_BITMASK(48,63)
-
-#define XSCOM_DATA_IND_READ		PPC_BIT(0)
-#define XSCOM_DATA_IND_COMPLETE		PPC_BIT(32)
-#define XSCOM_DATA_IND_ERR		PPC_BITMASK(33,35)
-#define XSCOM_DATA_IND_DATA		PPC_BITMASK(48,63)
-
-/* HB folks say: try 10 time for now */
-#define XSCOM_IND_MAX_RETRIES		10
-
 DEFINE_LOG_ENTRY(OPAL_RC_XSCOM_RW, OPAL_PLATFORM_ERR_EVT, OPAL_XSCOM,
 		OPAL_CEC_HARDWARE, OPAL_PREDICTIVE_ERR_GENERAL,
 		OPAL_NA);
@@ -165,12 +153,12 @@ static void xscom_handle_ind_error(uint64_t data, uint32_t gcid,
 	/* XXX: Create error log entry ? */
 	if (timeout)
 		log_simple_error(&e_info(OPAL_RC_XSCOM_INDIRECT_RW),
-			"XSCOM: %s indirect timeout, gcid=0x%x pcb_addr=0x%llx"
+			"XSCOM: indirect %s timeout, gcid=0x%x pcb_addr=0x%llx"
 			" stat=0x%x\n",
 			is_write ? "write" : "read", gcid, pcb_addr, stat);
 	else
 		log_simple_error(&e_info(OPAL_RC_XSCOM_INDIRECT_RW),
-			"XSCOM: %s indirect error, gcid=0x%x pcb_addr=0x%llx"
+			"XSCOM: indirect %s error, gcid=0x%x pcb_addr=0x%llx"
 			" stat=0x%x\n",
 			is_write ? "write" : "read", gcid, pcb_addr, stat);
 }
@@ -323,7 +311,7 @@ static int xscom_indirect_write(uint32_t gcid, uint64_t pcb_addr, uint64_t val)
 		if ((data & XSCOM_DATA_IND_COMPLETE) ||
 		    (retries >= XSCOM_IND_MAX_RETRIES)) {
 			xscom_handle_ind_error(data, gcid, pcb_addr,
-					       false);
+					       true);
 			rc = OPAL_HARDWARE;
 			goto bail;
 		}
