@@ -802,6 +802,7 @@ extern void fsp_ipmi_init(void);
 /* Reset/Reload */
 extern void fsp_reinit_fsp(void);
 extern void fsp_trigger_reset(void);
+extern void fsp_reset_links(void);
 
 /* FSP memory errors */
 extern void fsp_memory_err_init(void);
@@ -823,8 +824,12 @@ extern void fsp_epow_init(void);
 
 /* DPO */
 extern void fsp_dpo_init(void);
+extern bool fsp_dpo_pending;
 
 /* Chiptod */
 extern void fsp_chiptod_init(void);
 
+/* Terminate immediate */
+extern void __attribute__((noreturn)) ibm_fsp_terminate(const char *msg);
+
 #endif /* __FSP_H */
