@@ -144,6 +144,14 @@
 #define IPMI_MAX_REQ_SIZE		60
 #define IPMI_MAX_RESP_SIZE		60
 
+/*
+ * As far as I can tell the size of PEL record is unbounded (due to
+ * the possible presence of the user defined section). We chose this
+ * size because it's what hostboot also uses and most of the OPAL logs
+ * are few hundred bytes.
+ */
+#define IPMI_MAX_PEL_SIZE		0x800
+
 struct ipmi_backend;
 struct ipmi_msg {
 	/* Can be used by command implementations to track requests */
@@ -230,6 +238,9 @@ int ipmi_set_fw_progress_sensor(uint8_t state);
 /* Register a backend with the ipmi core. Currently we only support one. */
 void ipmi_register_backend(struct ipmi_backend *backend);
 
+/* Allocate IPMI SEL panic message */
+void ipmi_sel_init(void);
+
 /* Register rtc ipmi commands with as opal callbacks. */
 void ipmi_rtc_init(void);
 
@@ -265,4 +276,7 @@ uint8_t ipmi_get_sensor_number(uint8_t sensor_type);
 /* Set the boot count once the OS is up and running */
 int ipmi_set_boot_count(void);
 
+/* Terminate immediate */
+void __attribute__((noreturn)) ipmi_terminate(const char *msg);
+
 #endif
