@@ -91,13 +91,13 @@
 #define PSIHB_XIVR_LPC			0x70
 #define PSIHB_XIVR_LOCAL_ERR		0x78
 #define PSIHB_XIVR_HOST_ERR		0x80
-#define PSIHB_ISRN			0x88
-#define PSIHB_ISRN_COMP			PPC_BITMASK(0,18)
-#define PSIHB_ISRN_IRQ_MUX		PPC_BIT(28)
-#define PSIHB_ISRN_IRQ_RESET		PPC_BIT(29)
-#define PSIHB_ISRN_DOWNSTREAM_EN	PPC_BIT(30)
-#define PSIHB_ISRN_UPSTREAM_EN		PPC_BIT(31)
-#define PSIHB_ISRN_MASK			PPC_BITMASK(32,50)
+#define PSIHB_IRSN			0x88
+#define PSIHB_IRSN_COMP			PPC_BITMASK(0,18)
+#define PSIHB_IRSN_IRQ_MUX		PPC_BIT(28)
+#define PSIHB_IRSN_IRQ_RESET		PPC_BIT(29)
+#define PSIHB_IRSN_DOWNSTREAM_EN	PPC_BIT(30)
+#define PSIHB_IRSN_UPSTREAM_EN		PPC_BIT(31)
+#define PSIHB_IRSN_MASK			PPC_BITMASK(32,50)
 
 #define PSIHB_IRQ_STATUS		0x58
 #define   PSIHB_IRQ_STAT_OCC		PPC_BIT(27)
