@@ -56,6 +56,7 @@ static inline bool is_rodata(const void *p)
 	return ((const char *)p >= __rodata_start && (const char *)p < __rodata_end);
 }
 
+#define OPAL_BOOT_COMPLETE 0x1
 /* Debug descriptor. This structure is pointed to by the word at offset
  * 0x80 in the sapphire binary
  */
@@ -65,7 +66,7 @@ struct debug_descriptor {
 	u32	version;
 	u8	console_log_levels;	/* high 4 bits in memory,
 					 * low 4 bits driver (e.g. uart). */
-	u8	reserved1;
+	u8	state_flags; /* various state flags - OPAL_BOOT_COMPLETE etc */
 	u16	reserved2;
 	u32	reserved[2];
 
@@ -117,16 +118,6 @@ void _prlog(int log_level, const char* fmt, ...) __attribute__((format (printf,
 /* Location codes  -- at most 80 chars with null termination */
 #define LOC_CODE_SIZE	80
 
-enum ipl_state {
-	ipl_initial		= 0x00000000,
-	ipl_opl_sent		= 0x00000001,
-	ipl_got_continue	= 0x00000002,
-	ipl_got_new_role	= 0x00000004,
-	ipl_got_caps		= 0x00000008,
-	ipl_got_fsp_functional	= 0x00000010
-};
-extern enum ipl_state ipl_state;
-
 /* Processor generation */
 enum proc_gen {
 	proc_gen_unknown,
@@ -263,6 +254,12 @@ extern void *create_dtb(const struct dt_node *root);
 /* SLW reinit function for switching core settings */
 extern int64_t slw_reinit(uint64_t flags);
 
+/* SLW update timer function */
+extern void slw_update_timer_expiry(uint64_t new_target);
+
+/* Is SLW timer available ? */
+extern bool slw_timer_ok(void);
+
 /* Fallback fake RTC */
 extern void fake_rtc_init(void);
 
