@@ -25,11 +25,10 @@ unsigned long int strtoul(const char *nptr, char **endptr, int base);
 long int strtol(const char *nptr, char **endptr, int base);
 
 int rand(void);
-void __attribute__((noreturn)) _abort(void);
-#define abort() do {                                           \
-               update_sp_attn_area("abort():" __FILE__         \
-                                   ":" stringify(__LINE__));   \
-               _abort();                                       \
+void __attribute__((noreturn)) _abort(const char *msg);
+#define abort() do {					\
+               _abort("abort():" __FILE__		\
+		      ":" stringify(__LINE__));		\
        } while(0)
 
 
