@@ -267,9 +267,8 @@ static int flash_erase(struct blocklevel_device *bl, uint32_t dst, uint32_t size
 	return 0;
 }
 
-int flash_erase_chip(struct blocklevel_device *bl)
+int flash_erase_chip(struct flash_chip *c)
 {
-	struct flash_chip *c = container_of(bl, struct flash_chip, bl);
 	struct spi_flash_ctrl *ct = c->ctrl;
 	int rc;
 
@@ -692,11 +691,10 @@ static int flash_set_4b(struct flash_chip *c, bool enable)
 	return ct->cmd_wr(ct, enable ? CMD_EN4B : CMD_EX4B, false, 0, NULL, 0);
 }
 
-int flash_force_4b_mode(struct blocklevel_device *bl, bool enable_4b)
+int flash_force_4b_mode(struct flash_chip *c, bool enable_4b)
 {
-	struct flash_chip *c = container_of(bl, struct flash_chip, bl);
 	struct spi_flash_ctrl *ct = c->ctrl;
-	int rc;
+	int rc = FLASH_ERR_4B_NOT_SUPPORTED;
 
 	/*
 	 * We only allow force 4b if both controller and flash do 4b
@@ -704,7 +702,7 @@ int flash_force_4b_mode(struct blocklevel_device *bl, bool enable_4b)
 	 * access a direct mapped read region
 	 */
 	if (enable_4b && !((c->info.flags & FL_CAN_4B) && ct->set_4b))
-		return FLASH_ERR_4B_NOT_SUPPORTED;
+		return rc;
 
 	/* Only send to flash directly on controllers that implement
 	 * the low level callbacks
@@ -798,7 +796,8 @@ static int flash_get_info(struct blocklevel_device *bl, const char **name,
 	return 0;
 }
 
-int flash_init(struct spi_flash_ctrl *ctrl, struct blocklevel_device **bl)
+int flash_init(struct spi_flash_ctrl *ctrl, struct blocklevel_device **bl,
+		struct flash_chip **flash_chip)
 {
 	struct flash_chip *c;
 	int rc;
@@ -842,6 +841,9 @@ bail:
 	c->bl.flags = WRITE_NEED_ERASE;
 
 	*bl = &(c->bl);
+	if (flash_chip)
+		*flash_chip = c;
+
 	return 0;
 }
 
@@ -852,3 +854,11 @@ void flash_exit(struct blocklevel_device *bl)
 		free(container_of(bl, struct flash_chip, bl));
 }
 
+void flash_exit_close(struct blocklevel_device *bl, void (*close)(struct spi_flash_ctrl *ctrl))
+{
+	if (bl) {
+		struct flash_chip *c = container_of(bl, struct flash_chip, bl);
+		close(c->ctrl);
+		free(c);
+	}
+}
