@@ -115,6 +115,7 @@ void astbmc_init(void)
 
 	/* Register the BT interface with the IPMI layer */
 	bt_init();
+	ipmi_sel_init();
 	ipmi_wdt_init();
 	ipmi_rtc_init();
 	ipmi_opal_init();
@@ -342,9 +343,11 @@ void astbmc_early_init(void)
 		ast_setup_sio_uart1(UART_IO_BASE, UART_LPC_IRQ);
 	}
 
-	/* Similarily, some BMCs don't configure the BT interrupt properly */
+	/* Similarly, some BMCs don't configure the BT interrupt properly */
 	ast_setup_ibt(BT_IO_BASE, BT_LPC_IRQ);
 
 	/* Setup UART and use it as console with interrupts */
 	uart_init(true);
+
+	prd_init();
 }
