@@ -39,8 +39,6 @@ static bool palmetto_probe(void)
 	/* Lot of common early inits here */
 	astbmc_early_init();
 
-	prd_init();
-
 	return true;
 }
 
@@ -56,4 +54,5 @@ DECLARE_PLATFORM(palmetto) = {
 	.start_preload_resource	= flash_start_preload_resource,
 	.resource_loaded	= flash_resource_loaded,
 	.exit			= ipmi_wdt_final_reset,
+	.terminate		= ipmi_terminate,
 };
