@@ -24,29 +24,33 @@ if [ ! `command -v expect` ]; then
     exit 0;
 fi
 
-
-export SKIBOOT_ZIMAGE=`pwd`/zImage.epapr
+if [ -z "$SKIBOOT_ZIMAGE" ]; then
+    export SKIBOOT_ZIMAGE=`pwd`/zImage.epapr
+fi
 
 if [ ! -f "$SKIBOOT_ZIMAGE" ]; then
-    echo 'No zImage.epapr, skipping boot test';
+    echo "No $SKIBOOT_ZIMAGE, skipping boot test";
     exit 0;
 fi
 
+if [ -z "$SKIBOOT_MEM_DUMP" ]; then
+    export SKIBOOT_MEM_DUMP=skiboot-boot_test.dump
+fi
+
 # Currently getting some core dumps from mambo, so disable them!
 OLD_ULIMIT_C=`ulimit -c`
 ulimit -c 0
 
-( cd external/mambo; 
+( cd external/mambo;
 cat <<EOF | expect
 set timeout 600
-spawn $MAMBO_PATH/$MAMBO_BINARY -n -f skiboot.tcl
+spawn $MAMBO_PATH/$MAMBO_BINARY -n -f ../../test/run_boot_test.tcl
 expect {
 timeout { send_user "\nTimeout waiting for petitboot\n"; exit 1 }
 eof { send_user "\nUnexpected EOF\n;" exit 1 }
-"Welcome to Petitboot"
+"Execution stopped: Sim Support exit requested stop"
 }
-
-close
+wait
 exit 0
 EOF
 )
